@@ -1,6 +1,6 @@
 import os, time
 from os.path import *
-import gnomeapplet, gtk, gtk.gdk, gconf, gobject
+import gnomeapplet, gtk, gtk.gdk, gconf, gnomevfs, gobject
 from gettext import gettext as _
 
 import deskbar, deskbar.ui
@@ -15,6 +15,9 @@ from deskbar.ui.cuemiac.CuemiacButtonUI import CuemiacButtonUI
 from deskbar.ui.entriac.CuemiacEntryUI import CuemiacEntryUI
 from deskbar.ui.window.CuemiacWindowUI import CuemiacWindowUI
 
+PROXY_USE_HTTP_PROXY = '/system/http_proxy/use_http_proxy'
+PROXY_HOST_KEY = '/system/http_proxy/host'
+PROXY_PORT_KEY = '/system/http_proxy/port'
 
 class DeskbarApplet:
 	def __init__(self, applet):
@@ -57,7 +60,18 @@ class DeskbarApplet:
 		if self.clear_entry == None:
 			self.clear_entry = False
 		deskbar.GCONF_CLIENT.notify_add(self.prefs.GCONF_CLEAR_ENTRY, lambda x, y, z, a: self.on_clear_entry_changed (z.value))
-		
+	
+		self.use_proxy = deskbar.GCONF_CLIENT.get_bool(PROXY_USE_HTTP_PROXY)
+		if self.use_proxy == None:
+			self.use_proxy = False
+		deskbar.GCONF_CLIENT.notify_add(PROXY_USE_HTTP_PROXY, lambda x, y, z, a: self.on_use_proxy_changed (z.value))
+
+		self.proxy_host = deskbar.GCONF_CLIENT.get_string(PROXY_HOST_KEY)
+		deskbar.GCONF_CLIENT.notify_add(PROXY_HOST_KEY, lambda x, y, z, a: self.on_proxy_host_changed (z.value))
+
+		self.proxy_port = deskbar.GCONF_CLIENT.get_int(PROXY_PORT_KEY)
+		deskbar.GCONF_CLIENT.notify_add(PROXY_PORT_KEY, lambda x, y, z, a: self.on_proxy_port_changed (z.value))
+
 		# Watch out for UI override from command line
 		if deskbar.UI_OVERRIDE:
 			ui_name = deskbar.UI_OVERRIDE
@@ -77,7 +91,11 @@ class DeskbarApplet:
 			self.ui.connect('show-preferences', self.on_preferences, None)
 			self.ui.connect('show-about', self.on_about, None)
 			self.ui.connect('clear-history', self.on_clear_history, None)
-		
+		else:
+			# How can this happen ?? Apparently a bug report had this problem
+			# I check and recheck, i can't figure out a way that ui_name is unset..
+			self.ui = CuemiacButtonUI (applet, self.prefs)
+			
 		# Set up the chosen UI
 		self.set_up_ui_signals ()
 		self.ui.set_sensitive (False)
@@ -122,6 +140,9 @@ class DeskbarApplet:
 		get_deskbar_history().add(text, match)
 		if self.clear_entry:
 			gobject.idle_add(self.do_clear_entry)
+		else:
+			self.ui.get_entry().set_position(-1)
+			self.ui.get_entry().select_region(0, -1)
 	
 	def do_clear_entry(self):
 		entry = self.ui.get_entry()
@@ -208,7 +229,7 @@ class DeskbarApplet:
 				break
 					
 	def on_about (self, component, verb):
-		show_about()
+		show_about(self.applet)
 	
 	def on_preferences (self, component, verb):
 		show_preferences(self, self.loader, self.module_list)
@@ -292,6 +313,12 @@ class DeskbarApplet:
 	
 	def on_applet_button_release(self, widget, event):
 		if event.button == 2:
+			# The check for middle_click != None is to fix bug 355687
+			# this happens when the mouse is middle_clicked then moved inside
+			# the entry and released. In this case we ignore it.
+			if self.middle_click is None:
+				return
+				
 			x, y = self.middle_click
 			diffx, diffy = abs(x-event.x_root), abs(y-event.y_root)
 			if diffx <= 3 and diffy <=3:
@@ -369,4 +396,18 @@ class DeskbarApplet:
 	def on_use_selection(self, value=None):
 		if value != None and value.type == gconf.VALUE_BOOL:
 			self.use_selection = value.get_bool()
+
+	def on_use_proxy_changed(self, value):
+		if value is None or value.type != gconf.VALUE_BOOL:
+			return
+		self.use_proxy = value.get_bool()
 	
+	def on_proxy_host_changed(self,value):
+		if value is None or value.type != gconf.VALUE_STRING:
+			return
+		self.proxy_host = value.get_string()
+
+	def on_proxy_port_changed(self,value):
+		if value is None or value.type != gconf.VALUE_INT:
+			return
+		self.proxy_port = value.get_int()
