@@ -1,12 +1,19 @@
 from gettext import gettext as _
-from os.path import join
+from os.path import join, isdir
+import os
 import struct
 import gtk, gtk.gdk, gtk.glade, gobject, gconf
+import dbus
 import deskbar, deskbar.Utils
-from deskbar.ui.ModuleListView import ModuleListView
+from deskbar.updater.NewStuffUpdater import NewStuffUpdater
+from deskbar.ui.ModuleListView import ModuleListView, WebModuleListView
+from deskbar.ModuleList import WebModuleList
 from deskbar import CUEMIAC_UI_NAME, ENTRIAC_UI_NAME, WINDOW_UI_NAME
 from deskbar.ModuleInstaller import ModuleInstaller
 
+if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
+	import dbus.glib
+
 MAXINT = 2 ** ((8 * struct.calcsize('i')) - 1) - 1
 
 class AccelEntry( gobject.GObject ):
@@ -16,15 +23,21 @@ class AccelEntry( gobject.GObject ):
 						 [gobject.TYPE_STRING, gobject.TYPE_UINT, gobject.TYPE_UINT, gobject.TYPE_UINT]),
 	}
 	__gproperties__ = {
-		'message': (gobject.TYPE_STRING, 'Prompt message', 'Message that prompts the user to assign a new accelerator', _('New accelerator...'), gobject.PARAM_READWRITE|gobject.PARAM_CONSTRUCT),
 		'accel_key': ( gobject.TYPE_UINT, "Accelerator key", "Accelerator key", 0, MAXINT, 0, gobject.PARAM_READWRITE ),
-		'accel_mask': ( gobject.TYPE_FLAGS, "Accelerator modifiers", "Accelerator modifiers", 0, gobject.PARAM_READWRITE ),
+		'accel_mods': ( gobject.TYPE_FLAGS, "Accelerator modifiers", "Accelerator modifiers", 0, gobject.PARAM_READWRITE ),
 		'keycode': ( gobject.TYPE_UINT, "Accelerator keycode", "Accelerator keycode", 0, MAXINT, 0, gobject.PARAM_READWRITE ),
 	}
+	
+	FORBIDDEN_KEYS = (gtk.keysyms.BackSpace, gtk.keysyms.Begin, gtk.keysyms.Delete, gtk.keysyms.End, gtk.keysyms.Down,
+					  gtk.keysyms.Home, gtk.keysyms.Insert, gtk.keysyms.Left, gtk.keysyms.Page_Down, gtk.keysyms.Page_Up,
+					  gtk.keysyms.Right, gtk.keysyms.Tab, gtk.keysyms.Up, gtk.keysyms.bar, gtk.keysyms.braceleft,
+					  gtk.keysyms.braceright, gtk.keysyms.colon, gtk.keysyms.comma, gtk.keysyms.dollar, gtk.keysyms.equal,
+					  gtk.keysyms.exclam, gtk.keysyms.hyphen, gtk.keysyms.period, gtk.keysyms.percent, gtk.keysyms.question,
+					  gtk.keysyms.quotedbl, gtk.keysyms.semicolon, gtk.keysyms.slash, gtk.keysyms.space, gtk.keysyms.underscore)
 
 	def __init__(self, accel_name=''):
 		self.__old_value = None
-		self._attributes = {'accel_key': 0, 'accel_mask': 0, 'keycode': 0}
+		self._attributes = {'accel_key': 0, 'accel_mods': 0, 'keycode': 0}
 		gobject.GObject.__init__(self)
 		
 		self.entry = gtk.Entry()
@@ -36,15 +49,13 @@ class AccelEntry( gobject.GObject ):
 		self.set_accelerator_name(accel_name)
 
 	def do_get_property(self, pspec):
-		if pspec.name in ('message', 'accel_key', 'accel_mask', 'keycode'):
+		if pspec.name in ('accel_key', 'accel_mods', 'keycode'):
 			return self._attributes[pspec.name]
 	
 	def do_set_property(self, pspec, value):
-		if pspec.name == 'message':
-			self._attributes['message'] = value
-		elif pspec.name == 'accel_key':
+		if pspec.name == 'accel_key':
 			self.set_accelerator(int(value), self.get_property('keycode'), self.get_property('accel_mask'))
-		elif pspec.name == 'accel_mask':
+		elif pspec.name == 'accel_mods':
 			self.set_accelerator(self.get_property('accel_key'), self.get_property('keycode'), int(value))
 		elif pspec.name == 'keycode':
 			self.set_accelerator(self.get_property('accel_key'), int(value), self.get_property('accel_mask'))
@@ -62,9 +73,9 @@ class AccelEntry( gobject.GObject ):
 		return
 
 	def get_accelerator(self):
-		return ( self.get_property('accel_key'), self.get_property('keycode'), self.get_property('accel_mask') )
+		return ( self.get_property('accel_key'), self.get_property('keycode'), self.get_property('accel_mods') )
 
-	def set_accelerator(self, keyval, keycode, mask):
+	def set_accelerator(self, keyval, mods, keycode):
 		changed = False
 		self.freeze_notify()
 		if keyval != self._attributes['accel_key']:
@@ -72,9 +83,9 @@ class AccelEntry( gobject.GObject ):
 			self.notify('accel_key')
 			changed = True
 			
-		if mask != self._attributes['accel_mask']:
-			self._attributes['accel_mask'] = mask
-			self.notify('accel_mask')
+		if mods != self._attributes['accel_mods']:
+			self._attributes['accel_mods'] = mods
+			self.notify('accel_mods')
 			changed = True
 			
 		if keycode != self._attributes['keycode']:
@@ -84,53 +95,86 @@ class AccelEntry( gobject.GObject ):
 			
 		self.thaw_notify()
 		if changed:
-			text = self.__convert_keysym_state_to_string (keyval, keycode, mask)
+			text = self.__convert_keysym_state_to_string (keyval, keycode, mods)
 			self.entry.set_text(text)
 			
-	def __convert_keysym_state_to_string(self, keysym, keycode, mask):
-		if (keysym != 0 and keycode != 0):
-			return gtk.accelerator_name(keysym, mask)
+	def __convert_keysym_state_to_string(self, keysym, keycode, mask):		
+		name = gtk.accelerator_name(keysym, mask)
+		if keysym == 0:
+			name = "%s0x%02x" % (name, keycode)
+		return name
 
 	def get_widget(self):
 		return self.entry
 
 	def __on_button_press_event(self, entry, event):
 		self.__old_value = self.entry.get_text()
-		entry.set_text(self.get_property('message'))
+		entry.set_text( _('New accelerator...') )
 		entry.grab_focus()
 		return True
 
 	def __on_key_press_event(self, entry, event):
+		accel_mods = 0
 		edited = False
 		
 		keymap = gtk.gdk.keymap_get_default()
 		translation = keymap.translate_keyboard_state(event.hardware_keycode, event.state, event.group)
 		if translation == None:
-			self.__revert()
-			return
+			consumed_modifiers = 0
+		else:
+			(keyval, egroup, level, consumed_modifiers) = translation
+		
+		accel_keyval = gtk.gdk.keyval_to_lower(event.keyval)
+		if (accel_keyval == gtk.keysyms.ISO_Left_Tab):
+		    accel_keyval = gtk.keysyms.Tab
+		
+		accel_mods = event.state & gtk.accelerator_get_default_mod_mask()
+		
+		# Filter consumed modifiers		
+		accel_mods &= ~consumed_modifiers
+		
+  		# Put shift back if it changed the case of the key, not otherwise.
+		if (accel_keyval != event.keyval):
+			accel_mods |= gtk.gdk.SHIFT_MASK
 			
-		(keyval, egroup, level, consumed_modifiers) = translation
-		upper = event.keyval
-		accel_keyval = gtk.gdk.keyval_to_lower(upper)
-
-		# Put shift back if it changed the case of the key, not otherwise.
-		if upper != accel_keyval and (consumed_modifiers & gtk.gdk.SHIFT_MASK):
-			consumed_modifiers &= ~(gtk.gdk.SHIFT_MASK)
-
-		# filter consumed/ignored modifiers
-		ignored_modifiers = gtk.gdk.MOD2_MASK | gtk.gdk.MOD5_MASK
-		accel_mods = event.state & gtk.gdk.MODIFIER_MASK & ~(consumed_modifiers | ignored_modifiers)
-		
-		if accel_mods == 0 and accel_keyval == gtk.keysyms.Escape:
+		if accel_mods == 0:
+			if accel_keyval == gtk.keysyms.Escape:
+				self.__revert()
+				return
+						
+		# Do not make keyboard unusable
+		if ( ((accel_mods == 0 or accel_mods == gtk.gdk.SHIFT_MASK) and accel_keyval >= gtk.keysyms.a and accel_keyval <= gtk.keysyms.z) # alphabet
+			or (accel_mods == 0 and (
+				   (accel_keyval >= 48 and accel_keyval <= 57) # number keys
+				or (accel_keyval >= gtk.keysyms.kana_fullstop and accel_keyval <= gtk.keysyms.semivoicedsound)
+	   			or (accel_keyval >= gtk.keysyms.Arabic_comma and accel_keyval <= gtk.keysyms.Arabic_sukun)
+	   	 		or (accel_keyval >= gtk.keysyms.Serbian_dje and accel_keyval <= gtk.keysyms.Cyrillic_HARDSIGN)
+	   	  		or (accel_keyval >= gtk.keysyms.Greek_ALPHAaccent and accel_keyval <= gtk.keysyms.Greek_omega)
+	   	   		or (accel_keyval >= gtk.keysyms.hebrew_doublelowline and accel_keyval <= gtk.keysyms.hebrew_taf)
+   	   			or (accel_keyval >= gtk.keysyms.Thai_kokai and accel_keyval <= gtk.keysyms.Thai_lekkao)
+   	   			or (accel_keyval >= gtk.keysyms.Hangul and accel_keyval <= gtk.keysyms.Hangul_Special)
+   	   			or (accel_keyval >= gtk.keysyms.Hangul_Kiyeog and accel_keyval <= gtk.keysyms.Hangul_J_YeorinHieuh)
+   	   		))
+			or (accel_keyval in self.FORBIDDEN_KEYS)
+			or (accel_keyval == 0 and accel_mods != 0) ):
+			dialog = gtk.MessageDialog (self.entry.get_toplevel(),
+				gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL,
+				gtk.MESSAGE_WARNING,
+				gtk.BUTTONS_CANCEL,
+				_("The shortcut \"%s\" cannot be used because it will become unusable to type using this key.\nPlease try with a key such as Control, Alt or Shift at the same time.\n")
+				% gtk.accelerator_name(accel_keyval, accel_mods)
+			)
+			dialog.run()
+			dialog.destroy()
+			self.__revert()
+			return
+						
+		if not gtk.accelerator_valid(accel_keyval, accel_mods):
 			self.__revert()
 			return
 		
-		if not gtk.accelerator_valid(accel_keyval, accel_mods):
-			self.__cancel()
-			return
-
-		accel_name = gtk.accelerator_name(accel_keyval, accel_mods)
-		self.set_accelerator(accel_keyval, event.hardware_keycode, accel_mods)
+		accel_name = self.__convert_keysym_state_to_string(accel_keyval, event.hardware_keycode, accel_mods)
+		self.set_accelerator(accel_keyval, accel_mods, event.hardware_keycode)
 		self.__old_value = None
 		self.emit('accel-edited', accel_name, accel_keyval, accel_mods, event.hardware_keycode)
 		return True
@@ -138,9 +182,6 @@ class AccelEntry( gobject.GObject ):
 	def __on_focus_out_event(self, entry, event):
 		if self.__old_value != None:
 			self.__revert()
-
-	def __cancel(self):
-		self.set_accelerator_name("")
 	
 	def __revert(self):
 		self.set_accelerator_name(self.__old_value)
@@ -148,11 +189,15 @@ class AccelEntry( gobject.GObject ):
 class DeskbarPreferencesUI:
 	def __init__(self, applet, module_loader, module_list):
 		self.module_list = module_list
+		self.web_module_list = WebModuleList()
 		self.module_loader = module_loader
 		self.glade = gtk.glade.XML(join(deskbar.SHARED_DATA_DIR, "prefs-dialog.glade"))
 		
 		self.dialog = self.glade.get_widget("preferences")
 		
+		# Since newstuff is optional we have to check if self.newstuff is None each time we use it
+		self.newstuff = None
+			
 		# Retreive current values
 		self.width = deskbar.GCONF_CLIENT.get_int(applet.prefs.GCONF_WIDTH)
 		self.expand = deskbar.GCONF_CLIENT.get_bool(applet.prefs.GCONF_EXPAND)
@@ -211,6 +256,9 @@ class DeskbarPreferencesUI:
 		self.use_selection_box.connect('toggled', self.on_use_selection_toggled, applet)
 		self.use_selection_id = deskbar.GCONF_CLIENT.notify_add(applet.prefs.GCONF_USE_SELECTION, lambda x, y, z, a: self.on_config_use_selection(z.value))
 		
+		# Setup new-stuff-manager
+		self.__enable_newstuffmanager( self.__is_nsm_available() )
+  		
 		# Setup Drag & Drop
 		big_box = self.glade.get_widget("big_box")
 		self.TARGET_URI_LIST, self.TARGET_NS_URL = range(2)
@@ -225,7 +273,66 @@ class DeskbarPreferencesUI:
 		big_box.connect("drag_leave", self.on_drag_leave)
 		
 		self.sync_ui()
-		
+
+	def __invoke_newstuff(self, func):
+  		if self.newstuff == None:			
+  			self.newstuff = NewStuffUpdater(self.dialog, self.module_loader, self.module_list, self.web_module_list)
+  			self.newstuff.connect('ready', self.on_newstuff_ready, getattr(self.newstuff, func))
+  			self.newstuff.connect('connection-failed', self.on_connection_failed)			
+  		else:
+  			getattr(self.newstuff, func)()
+
+	def __is_nsm_available(self):
+		bus = dbus.SessionBus()
+		proxy = bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
+		_dbus = dbus.Interface(proxy, 'org.freedesktop.DBus')
+		_dbus.ReloadConfig()
+		bus_names = _dbus.ListActivatableNames()
+		return (NewStuffUpdater.NEW_STUFF_SERVICE in bus_names)
+
+	def __enable_newstuffmanager(self, status):
+		if status:
+			container = self.glade.get_widget("newhandlers")
+  			self.webmoduleview = WebModuleListView(self.web_module_list)
+	  		self.webmoduleview.get_selection().connect("changed", self.on_webmodule_selected)
+  			self.web_module_list.connect('row-changed', lambda list, path, iter: self.on_webmodule_selected	(self.webmoduleview.get_selection()))
+	  		container.add(self.webmoduleview)
+	  		
+	  		self.install = self.glade.get_widget("install")
+	  		self.check_new_extensions = self.glade.get_widget("check_new_extensions")
+	  		self.check = self.glade.get_widget("check")
+	  		self.update = self.glade.get_widget("update")
+  		
+	  		self.check.connect('clicked', self.on_check_handlers)
+	  		self.update.connect('clicked', self.on_update_handler)
+	  		self.update.set_sensitive(False)
+	  		self.check_new_extensions.connect('clicked', self.on_check_new_extensions)
+	  		self.install.connect('clicked', self.on_install_handler)
+	  		self.install.set_sensitive(False)
+		else:
+			notebook = self.glade.get_widget("notebook1")
+			tab = self.glade.get_widget("extensions_vbox")
+			notebook.remove_page( notebook.page_num(tab) )
+			# Remove buttons in handlers tab
+			self.glade.get_widget("check").destroy()
+	  		self.glade.get_widget("update").destroy()
+  		
+  	def on_connection_failed(self, newstuff, error):
+  		"""
+  		Called if a connection to the repository failed
+  		
+  		An error message will be displayed in a MessageDialog
+  		and C{self.newstuff} is reset
+  		
+  		@type error: dbus_bindings.DBusException instance
+  		"""
+  		dialog = gtk.MessageDialog(parent=self.dialog, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)		
+  		dialog.set_markup('<span weight="bold" size="larger">%s</span>\n\n%s' % (_("Connection to repository failed"), _("Please check your internet connection")))
+  		dialog.run()
+  		dialog.destroy()
+  		self.newstuff.close()
+  		self.newstuff = None
+	
 	def show_run_hide(self, parent):
 		self.dialog.set_screen(parent.get_screen())
 		self.dialog.show_all()
@@ -234,6 +341,8 @@ class DeskbarPreferencesUI:
 	
 	def on_dialog_response(self, dialog, response):	
 		self.dialog.destroy()
+		if self.newstuff != None:
+			self.newstuff.close()
 		
 		deskbar.GCONF_CLIENT.notify_remove(self.width_notify_id)
 		deskbar.GCONF_CLIENT.notify_remove(self.expand_notify_id)
@@ -343,7 +452,15 @@ class DeskbarPreferencesUI:
 		if module_context != None:
 			self.check_requirements(module_context)
 			gobject.timeout_add(1000, self.poll_requirements, module_context)
+			
+		# Check if we can update
+		if self.newstuff != None:
+			self.update.set_sensitive(module_context != None and module_context.update_infos[0])				
 	
+	def on_webmodule_selected(self, selection):
+		module_context = self.webmoduleview.get_selected_module_context()
+		self.install.set_sensitive(module_context != None and not module_context.installing)
+		
 	def poll_requirements(self, module_context):
 		try:
 			if module_context != self.moduleview.get_selected_module_context():
@@ -391,6 +508,32 @@ class DeskbarPreferencesUI:
 			loader.stop_module_async (context)
 		else:
 			loader.initialize_module_async (context)
+	
+	def on_newstuff_ready(self, newstuff, func):
+		func()
+		
+	def on_check_handlers(self, button):
+		#Update all handlers
+		self.__invoke_newstuff('check_all')
+			
+	def on_check_new_extensions(self, button):
+		self.__invoke_newstuff('check_new')
+		
+	def on_update_handler(self, button):
+		module_context = self.moduleview.get_selected_module_context()
+		if module_context != None:
+			# Trigger module update
+			if self.newstuff != None:
+				self.newstuff.update(module_context)
+			button.set_sensitive(False)
+		
+	def on_install_handler(self, button):
+		# Install the selected new handler
+		module_context = self.webmoduleview.get_selected_module_context()
+		if module_context != None:
+			if self.newstuff != None:
+				self.newstuff.install(module_context)
+			button.set_sensitive(False)
 			
 	def on_drag_motion(self, widget, drag_context, x, y, timestamp):
 		return False
